# DisclosureTriangle.tcl
# (C) 2005 reddog
# This program is placed in Tcl Licence. see license_term.
# http://www.tcl.tk/software/tcltk/license_terms.html

package require Tcl 8.4
package require Tk  8.4
package provide DisclosureTriangle 0.1

namespace eval DisclosureTriangle {
	namespace export DisclosureTriangle
	
	# Initialize OptionDatabase
	proc initOption {className option} {
		upvar $option Option
		foreach name [array names Option] {
			if {[string index $name 0] ne {-}} continue
			set optionName [lindex $Option($name) 0]
			set optionClass [lindex $Option($name) 1]
			set optionValue [lindex $Option($name) 2]
			option add *$className.$optionName $optionValue widgetDefault
		}
		return
	}
	
	# Solve a Alias Option Name
	proc solveAliasOptionName {defaultOption option} {
		upvar $defaultOption Option
		if {[llength $Option($option)] == 1} {
			return $Option($option)
		}
		return $option
	}
	
	# Get 1 or 0
	proc getBoolInt {v} {
		switch -- [string tolower $v] {
			1   {return 1}  0   {return 0}
			on  {return 1}  off {return 0}
			yes {return 1}  no  {return 0}
		}
		return 1
	}

	#-----------------------------------------------------------
	# Widget Specific Procs
	#-----------------------------------------------------------
	array set Common {
		className DisclosureTriangle
	}
	# Images
	set Common(imgLeft) [image create photo [namespace current]::arrowLeft -data {
		R0lGODlhEgAMAKIAAMzM/7u7u5mZ/4iIiGZmzAAAAP///wAAACH5BAUUAAYA
		LAAAAAASAAwAAAMnaLpW/FAVF2sBtL4rspaA0H0LJxCeFaJDQF6s+3YtKRW1
		3eS6IUcJADs=
	}]
	set Common(imgDown) [image create photo [namespace current]::arrowDown -data {
		R0lGODlhEgAMAKIAAMzM/7u7u5mZ/4iIiGZmzAAAAP///wAAACH5BAUUAAYA
		LAAAAAASAAwAAAMjaLrc/jBKVaq9BRbAuyhDoAmkQIBiVJBnOBlFm77oy8x2
		nicAOw==
	}]

	# Set Widget Default Options
	# if value is not list, that option is an alias of another option.
	# if first char is -, public option, and not, private option
	array set DefaultOption [list \
		-showstatus   [list showStatus		ShowStatus     			 0] \
		-showtext     [list showText		ShowText   		   Show...] \
		-hidetext     [list hideText		HideText  		   Hide...] \
		-width        [list width			Width					 0] \
		-w            -width \
		-separator    [list separator		Separator				 0] \
		-font         [list font			Font					{}] \
		rootw         {} \
	]
	initOption $Common(className) DefaultOption
	
	# Hidden Sub Options
#    option add *$Common(className).Button.background    yellow   widgetDefault
	
	# Create command
	proc DisclosureTriangle {w args} {
		variable Common
		variable DefaultOption
		
		if {[info command ${w}] ne {}} {
			error "window name \"$w\" already exists in parent"
		}
		
		
		# Create widget
		frame $w -class $Common(className)
		frame $w.t
		frame $w.fw -width [option get $w width Width]
		label $w.t.l -compound left -anchor w
		frame $w.t.s -relief flat -bd 2 -height 2
		frame $w.f
		
		pack $w.fw -side top
		pack $w.t -side top -fill x 
		pack $w.t.l -side left 
		pack $w.t.s -side left -fill x -expand yes
		if {[option get $w separator Separator]} {
			$w.t.s configure -relief groove
		}

		# Show Status
		if {[option get $w showStatus ShowStatus]} {
			$w.t.l configure -text [option get $w hideText HideText] -image $Common(imgDown)
			pack $w.f -side top -expand yes -fill both
		} else {
			$w.t.l configure -text [option get $w showText ShowText] -image $Common(imgLeft)
			pack forget $w.f
		}
		
		# Bind
		bind $w.t.l <1> "if {\[$w cget -showstatus\]} {$w hide} else {$w show}"
#        bind $Common(className).l <Enter> {tk_messageBox -message Enter}
		
		# Options
		array set [namespace current]::${w} {}
		foreach name [array names DefaultOption] {
			set optionName [lindex $DefaultOption($name) 0]
			set optionClass [lindex $DefaultOption($name) 1]
			set [namespace current]::${w}($name) \
				[option get $w $optionName $optionClass]
		}
		
		# Create Command
		rename ::$w [namespace current]::${w}::${w}
		set [namespace current]::${w}(rootw) [namespace current]::${w}::${w}
		proc ::${w} {command args} \
			"eval [namespace current]::dtCommand $w \$command \$args"
		
		
		# Destroy bind
		bind $w <Destroy> [namespace code {
			array unset [namespace current]::%W
			namespace delete [namespace current]::%W
			rename %W {}
		}]
		

		# Init
		
		if {[catch "dtCommand $w configure $args" ret]} {
			destroy $w
			error $ret
		}

		return $w
	}
	
	# Widget Commands
	proc dtCommand {w command args} {
		variable Common
		variable DefaultOption
		variable $w
		upvar 0 $w widgetOption

		switch -- $command {
			gettopframe {
				if {[llength $args] > 0} {
					error "wrong # args: should be \"$w gettopframe\""
				}
				return $w.t
			}

			getframe {
				if {[llength $args] > 0} {
					error "wrong # args: should be \"$w getframe\""
				}
				return $w.f
			}

			show {
				if {[llength $args] > 0} {
					error "wrong # args: should be \"$w show\""
				}
				pack $w.f -side top -expand yes -fill both
				$w.t.l configure -image $Common(imgDown) -text $widgetOption(-hidetext)
				wm geometry [winfo toplevel $w] {}
				set widgetOption(-showstatus) 1
			}

			hide {
				if {[llength $args] > 0} {
					error "wrong # args: should be \"$w hide\""
				}
				pack forget $w.f
				$w.t.l configure -image $Common(imgLeft) -text $widgetOption(-showtext)
				wm geometry [winfo toplevel $w] {}
				set widgetOption(-showstatus) 0
			}

			cget {
				if {[llength $args] != 1} {
					error "wrong # args: should be \"$w cget option\""
				}
				
				if {[array names widgetOption $args] eq {} ||
					[string index $args 0] ne {-}} {
					error "unknown option \"$args\""
				}
				set args [solveAliasOptionName DefaultOption $args]
				return $widgetOption($args)
			}
			
			configure {
				# show all options
				if {[llength $args] == 0} {
					set result [list]
					foreach name [array names DefaultOption] {
						if {[string index $name 0] ne {-}} continue
						lappend result [concat $name $DefaultOption($name) $widgetOption($name)]
					}
					return $result
				}
				# show a option
				if {[llength $args] == 1} {
					if {[array names DefaultOption $args] eq {} ||
						[string index $args 0] ne {-}} {
						error "unknown option \"$args\""
					}
					foreach name [array names DefaultOption $args] {
						set name [solveAliasOptionName DefaultOption $name]
						return [concat $name $DefaultOption($name) $widgetOption($name)]
					}
				}
				#check all and set options
				foreach {key value} $args {
					if {[array names DefaultOption $key] eq {} ||
						[string index $key 0] ne {-}} {
						error "unknown option \"$key\""
					}
				}

				foreach {key value} $args {
					set key [solveAliasOptionName DefaultOption $key]

					switch -- $key {
						-showstatus {
							set b [getBoolInt $value]
							if {$b == 1} {$w show} else {$w hide}
							set widgetOption(-showstatus) $b
						}

						-showtext {
							set widgetOption(-showtext) $value
							if {![$w cget -showstatus]} {
								$w.t.l configure -text $value
							}
						}

						-hidetext {
							set widgetOption(-hidetext) $value
							if {[$w cget -showstatus]} {
								$w.t.l configure -text $value
							}
						}

						-width {
							$w.fw configure -width $value
							set widgetOption(-width) $value
						}

						-separator {
							set b [getBoolInt $value]
							if $b {
								$w.t.s configure -relief groove
							} else {
								$w.t.s configure -relief flat
							}
							set widgetOption(-separator) $b
						}

						-font {
							$w.t.l configure -font $value
							set widgetOption(-font) $value
						}
					}
				}
				return
			}

			default {
				error "ambiguous command \"$command\": must be cget, configure, getframe, show, hide"
			}
		}
	}
	
}

